'****************************************************************
'*  Name    : Test_LED_uart1_26K20_LSM_V50.BAS                  *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 21/11/2017                                        *
'*  Version : 5.0                                               *
'*  Notes   : Using the Debounce library: DebounceInt.Inc       *
'*          : with the Amicus18 board.                          *
'*  Code powered by the Proton Development Suite Version 3603   *
'*          : This special code is used to check the bootloader *
'****************************************************************

'===============================================================================
' FOSC avaibled: 80, 64, 48, 40, 32, 25, 24, 20, 16, 14 (14.32MHz), 12, 10
' Header file for use with the Amicus18 board
' Same device as the Bootloader.
    Device = 18F26K20
    Declare Xtal = 64 

    ' DEFINE THE PLL: On / OFF
    $define PLL_ConfigFuses On

    ' Enable the to use the internal Oscillator.
    ' Uncomment the next line to enable the configuration.
    '$define _InternalOSC_

    ' Enable the Line to use the CLOCK OUT FUNCTION: 
    ' If CLKOUT function is enabled, CLKOUT on RA6 & Port function On RA7 otherwize Port function On RA6 & RA7.     
    '$Define _CLKOUT_Function_    

' (8) ENABLE THE PROTECTION OF THE BOOT BLOCK:
' Uncomment the next line to allow to protet the Boot Block against reading, writing, & Table read
    $define _EnableProtectBootBlock_
'===============================================================================
' Define the PORT & PIN of the test LED.
'
    $define _PORT PORTC
    $define _PIN 5
'===============================================================================
' COPY the Declares of the bootloader:
    Declare Optimiser_Level = 2
    Declare Dead_Code_Remove = On
    Declare Watchdog = On
    Declare Bootloader = OFF
    Declare PROTON_START_ADDRESS = 2048    
'===============================================================================
' Declare the UART Baud Rate manually: enable ONE line ONLY.

'    $define _BaudRate 9600

'    $define _BaudRate 19200
    
'    $define _BaudRate 38400
    
'    $define _BaudRate 57600
        
'    $define _BaudRate 115200

'-------------------------------------------------------------------------------
' Check if the Baudrate is correct for internal oscillator.
    $ifdef _InternalOSC_
        $if _xtal > 64
            $error "NO such FOSC for the internal oscillator!"
        $endif
        $if _BaudRate > 57600
        $undef _BaudRate
        $endif
    $endif
'-------------------------------------------------------------------------------
' The Declare UART Baudrate is automatic depending of the Xtal to be syncronized with the bootloader.
    $ifndef _BaudRate
        $if _xtal >= 40
            $ifdef _InternalOSC_
            $define _BaudRate 57600
            $else
            $define _BaudRate 115200
            $endif
        $endif
        $if (_xtal >= 16) And (_xtal < 40)
            $define _BaudRate 57600
        $endif
        $if _xtal < 16
            $define _BaudRate 38400
        $endif
    $endif
'===============================================================================
' Same Baudrate as the Bootloader to check the Software.
'-------------------------------------------------------------------------------
$if _BaudRate = 9600
    Declare Hserial_Baud = 9600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 19200
    Declare Hserial_Baud = 19200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 38400
    Declare Hserial_Baud = 38400
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 57600
    Declare Hserial_Baud = 57600
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
'-------------------------------------------------------------------------------
$if _BaudRate = 115200
$if _xtal = 80
' SYNC = 0, BRG16 = 1, BRGH = 1, BRG/EUSART Mode = 16-bit/Asynchronous
' Baud Rate Formula => FOSC/[4 (n+1)]
' Page 234 of the datasheet DS40001303H
' 
    Declare Hserial_TXSTA  = 36      ' Enable transmit, BRGH = 1
    Declare Hserial_RCSTA  = 144     ' Enable continuous receive 
    Declare Hserial_SPBRG  = 172     ' Baud Rate Generator Low Byte Value
    SPBRGH = 0                       ' Baud Rate Generator High Byte Value
    BAUDCONBits_BRG16 = 1            ' Enable the 16 bit Baud Rate Generator
    Declare Hserial_Clear  = On      ' Clear overflow automatically
$else
    Declare Hserial_Baud = 115200
    Declare Hserial_TXSTA = 36      ' Enable transmit and asynchronous mode
    Declare Hserial_RCSTA = 144     ' Enable serial port and continuous receive
    Declare Hserial_Clear = On      ' Enable Error clearing on received characters
$endif
$endif
'===============================================================================
' Same Config Fuses as Bootloader:
'
Config_Start
$if PLL_ConfigFuses = On
    $ifdef _InternalOSC_
        $if (_xtal = 32) Or (_xtal = 64)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = On"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 80)
        FOSC = HSPLL	;HS oscillator, PLL enabled (Clock Frequency = 4 x FOSC1)(Xtal piece 2,5 -> 20 MHz)
        $else
            $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = On"
        $endif
    $endif
$endif
$if PLL_ConfigFuses = OFF
    $ifdef _InternalOSC_
        $if (_xtal = 16)
            $ifdef CLKOUT_Function
            FOSC = INTIO7	;Internal oscillator block, CLKOUT in RA6, port function on RA7.        
            $else
            FOSC = INTIO67	;Internal oscillator block, port function on RA6 and RA7.
            $endif
        $else
            $error "Error in (Declare Xtal) with internal oscillator & PLL_ConfigFuses = OFF"
        $endif
    $else
        $if (_xtal >= 10) And (_xtal <= 25)
        FOSC = HS	    ;HS oscillator PLL disabled  (medium power 4-20 MHz) (Xtal piece 10 -> 25 MHz)
        $else
            $if (_xtal > 25) And (_xtal <= 80)
                $ifdef CLKOUT_Function
                FOSC = EC       ; External EC oscillator, CLKOUT function in RA6      
                $else
                FOSC = ECIO6    ; External EC oscillator, port function on RA6
                $endif
            $else
                $error "Error in (Declare Xtal) with external Xtal & PLL_ConfigFuses = OFF"
            $endif
       $endif
    $endif
$endif
    FCMEN = OFF	;Fail-Safe Clock Monitor disabled
    IESO = OFF	;Oscillator Switchover mode disabled
    PWRT = On	    ;PWRT enabled
    BOREN = On	;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 18	    ;VBOR set to 1.8 V nominal
    WDTEN = OFF	;WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 128	;1:128
    CCP2MX = PORTC	;CCP2 input/output is multiplexed with RC1
    PBADEN = On	;PORTB<4:0> pins are configured as analog input channels on Reset
    LPT1OSC = OFF	;Timer1 configured for higher power operation
    HFOFST = OFF	;The system clock is held off until the HFINTOSC is stable.
    MCLRE = On	;MCLR pin enabled; RE3 input pin disabled
    STVREN = On	;Stack full/underflow will cause Reset
    LVP = OFF	    ;Single-Supply ICSP disabled
    XINST = OFF	;Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    Debug = OFF	;Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
    Cp0 = OFF	    ;Block 0 (000800-003FFFh) not code-protected
    CP1 = OFF	    ;Block 1 (004000-007FFFh) not code-protected
    CP2 = OFF	    ;Block 2 (008000-00BFFFh) not code-protected
    CP3 = OFF	    ;Block 3 (00C000-00FFFFh) not code-protected
    $ifdef _EnableProtectBootBlock_
    CPB = On	    ;Boot block (000000-0007FFh) code-protected
    $else
    CPB = OFF	    ;Boot block (000000-0007FFh) not code-protected
    $endif    
    CPD = On	    ;Data EEPROM code-protected
    WRT0 = OFF	;Block 0 (000800-003FFFh) not write-protected
    WRT1 = OFF	;Block 1 (004000-007FFFh) not write-protected
    WRT2 = OFF	;Block 2 (008000-00BFFFh) not write-protected
    WRT3 = OFF	;Block 3 (00C000h-00FFFFh) not write-protected
    WRTC = On	    ;Configuration registers (300000-3000FFh) write-protected
    $ifdef _EnableProtectBootBlock_
    WRTB = On	    ;Boot Block (000000-0007FFh) write-protected
    $else
    WRTB = OFF	    ;Boot Block (000000-0007FFh) not write-protected
    $endif
    WRTD = OFF	;Data EEPROM not write-protected
    EBTR0 = OFF	;Block 0 (000800-003FFFh) not protected from table reads executed in other blocks
    EBTR1 = OFF	;Block 1 (004000-007FFFh) not protected from table reads executed in other blocks
    EBTR2 = OFF	;Block 2 (008000-00BFFFh) not protected from table reads executed in other blocks
    EBTR3 = OFF	;Block 3 (00C000-00FFFFh) not protected from table reads executed in other blocks
    $ifdef _EnableProtectBootBlock_
    EBTRB = On	;Boot Block (000000-0007FFh) protected from table reads executed in other blocks
    $else
    EBTRB = OFF	;Boot Block (000000-0007FFh) not protected from table reads executed in other blocks    
    $endif    
Config_End
'###############################################################################
'
' START THE USER SOFTWARE:
'
'###############################################################################
'===============================================================================
'    Include "D:\MyIncFiles\SYS_18Fxxk20.INC"
'    Include "SYS_18Fxxk20.INC"
'===============================================================================
' New Definitions of the library flags.
    Dim F_TMR2IF As PIR1Bits_TMR2IF     ' TMR2 To PR2 Match Interrupt Flag Bit
    Dim F_TMR2IE As PIE1Bits_TMR2IE     ' TMR2 To PR2 Match Interrupt Enable bi
    Dim F_TMR2ON As T2CONBits_TMR2ON    ' Timer2 On Bit 
    Dim F_IPEN As RCONBits_IPEN         ' Disable priority levels On interrupts (PIC16CXXX Compatibility mode)
    Dim F_TMR2IP As IPR1Bits_TMR2IP     ' TMR2 to PR2 Match Interrupt Priority bit
'------------------------------------------------------------------------------- 
'ASCII characters  
    Symbol Home = 1
    Symbol Bell = 7
    Symbol BkSp = 8
    Symbol Tab = 9
    Symbol CR = 13
    Symbol LF = $0A     ' Line Feed Char    
    Symbol True = 1     ' Boolean True
    Symbol False = 0    ' Boolean False
    Symbol Null = 0     ' Null character 
    Symbol ACK = 6
    Symbol STX = 2
    Symbol ETX = 3
    Symbol SPACE = $20
    Symbol DEL = 127    
    Symbol ESC = $1B    ' Escape
    Symbol BS = 08      ' Backspace 
'===============================================================================
' Declare some variables.
    Dim LoopByte As Byte
    Dim Temp1 As Byte
    Dim Temp2 As Byte
    Dim DataWord As Word
    Dim DataDword As Word    
    Dim EepAddress As Word
    Dim PICPartNumber As Word
    Dim PICRevision As Byte
    Dim PostScaler As Byte
    Dim CounterEep As Byte
    Dim NoASCII As Byte
    Dim IDLOCS_String As String * 8  
    
    Dim CodeTestFlags As Byte
    Dim F_LED8_On As CodeTestFlags.0
    Dim F_LED8_Off As CodeTestFlags.1    
    Dim F_LED9_On As CodeTestFlags.2
    Dim F_LED9_Off As CodeTestFlags.3
'===============================================================================
' Running with Internal oscillator ONLY.
' CONFIG FUSES:
' FOSC = INTIO67    ;Internal oscillator block, port function on RA6 and RA7
' HFOFST = OFF	    ;HFINTOSC output and ready status are delayed by the oscillator stable status
' Time for HF-INTOSC to Stabilize: 1mS, Time For PLL to Stabilize: 2mS
' Delay for code: 25mS (more or less)
'*******************************************************************************
'* To save code it is possible to disable theses lines of code,                *
'* because the Internal Oscillator is configurated by the Bootloader           *
'* or to change the HFINTOSC frequency                                         *
'* FOSC avaibled: 64, 48, 40, 32, 25, 24, 20, 16, 14 (14.32MHz), 12, 10        *
'*******************************************************************************
' Read the datasheet to change the FOSC frequency.
' The 8Mhz & 16Mhz outputs can use the PLL only.
'
    TBLPTRU = 0
$ifdef _InternalOSC_
    $if _xtal > 64
        $error "Error in (Declare Xtal) with internal oscillator"
    $endif
    ' See the Datasheet to configure the internal oscillator.
    '===========================================================================
INT_OSC:
    $if PLL_ConfigFuses = On
        $if _xtal = 64
        OSCCON = %01111000      ' 16Mhz x 4. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock. 
                                ' Bit 1 = 0, Bit 0 = 0, Primary clock (determined by FOSC<3:0> in CONFIG1H).
                                ' bit 3 OSTS: Oscillator Start-up Time-out Status bit
                                ' 1 = Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
        OSCTUNEBits_PLLEN = 1   ' PLL enabled (FOSC = 4 x INTOSC)
        $endif        
        $if _xtal = 32
        OSCCON = %01101000      ' 8Mhz x 4. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock. 
                                ' Bit 1 = 0, Bit 0 = 0, Primary clock (determined by FOSC<3:0> in CONFIG1H).
                                ' bit 3 OSTS: Oscillator Start-up Time-out Status bit
                                ' 1 = Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
        OSCTUNEBits_PLLEN = 1   ' PLL enabled (FOSC = 4 x INTOSC)
        $endif
    $endif 
    $if PLL_ConfigFuses = OFF
        $if _xtal = 16
        OSCCON = %01111000      ' 16Mhz x 1. Internal oscillator: Sleep, 16Mhz, R, R, Primary clock Direct. 
                                ' Bit 1 = 0, Bit 0 = 0, Primary clock (determined by FOSC<3:0> in CONFIG1H).
                                ' bit 3 OSTS: Oscillator Start-up Time-out Status bit
                                ' 1 = Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
        OSCTUNEBits_PLLEN = 0   ' PLL disabled (FOSC = 1 x INTOSC direct)
        $endif
    $endif
    '---------------------------------------------------------------------------
    OSCTUNEBits_INTSRC = 1              ' 1 = 31.25 kHz device clock derived from 16 MHz HFINTOSC source (divide-by-512 enabled) (if necessary)
    '---------------------------------------------------------------------------       
    $ifdef OSCCONBits_IOFS
    While OSCCONBits_IOFS = 0 : Wend    ' Wait untill OSC is stable (not a good job)
    $else
        $ifdef OSCCONBits_HFIOFS
        While OSCCONBits_HFIOFS = 0 : Wend  ' Wait untill OSC is stable (not a good job)
        $endif
    $endif    
    DelayMS 25
    '===========================================================================
$else
    '===========================================================================
    ' Oscillator Start-up Time-out Status bit:
    ' Device is running from the clock defined by FOSC<2:0> of the CONFIG1 register
    ' System Clock Select bits:
    ' Device is running from the clock defined by FOSC<3:0> of the CONFIG1H register
    OSCCON = %01111000       
    DelayMS 12
$endif    
'===============================================================================
    ' Definition file for number & name of PIC devices
    Include "D:\MyIncFiles\Devicelist.Inc"
'===============================================================================
    Include "D:\MyIncFiles\HRsout2StopBits.inc"
'===============================================================================
    $define ERASE_FLASH_BLOCKSIZE 64        ' For the PIC18F46K20.
'    Include "D:\MyIncFiles\ORG_SyncBlock.Inc"
'*******************************************************************************
'* Name    : On_Low_Interrupt                                                  *
'* Purpose : To test the ISR_Low Jump by the Bootloader.                       *
'*         : Clock = FOSC/4 => 64Mhz/4 = 16Mhz                                 *
'*******************************************************************************
    On_Low_Interrupt GoTo ISR_Low
    GoTo OverLowInterrupt
'===============================================================================   
ISR_Low:
    ' Timer is used to Toggle the LED9 every second.
    Context Save
    If INTCONBits_TMR0IF = 1 Then   ' Check the Timer0 Interrupt flag
        INTCONBits_TMR0IF = 0
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Toggle _PORT._PIN                 ' LED = One second ON, one second OFF.
    EndIf
    Context Restore
'===============================================================================
OverLowInterrupt:     
'===============================================================================
MAIN_TEST:
    #ifdef WatchDog_Req
    Clrwdt
    #endif
    DelayMS 50
'-------------------------------------------------------------------------------    
    INTCON2 = %10000000     ' PORTB Pullups disabled
    INTCON3 = 0
    PIR2 = 0
    PIE2 = 0
    IPR2 = 0                    
'
' Declare all digital   
    ADCON0 = 0    
    $ifdef _CM1CON0
    CM1CON0 = 0
    $endif
    $ifdef _CM2CON0
    CM2CON0 = 0
    $endif
    $ifdef _CM2CON1
    CM2CON1 = 0
    $endif
    $ifdef _ANSEL
    ANSEL = 0
    $endif
    $ifdef _ANSELH
    ANSELH = 0
    $endif
    $ifdef _ANSELA
    ANSELA = 0
    $endif
    $ifdef _ANSELB
    ANSELB = 0
    $endif
    $ifdef _ANSELC
    ANSELC = 0
    $endif
    $ifdef _ANSELD
    ANSELD = 0
    $endif    
' The eeprom is not used for the moment.
    EECON1 = 0

TRISPORT:
' PORTS
    ' TRIS the LED pin
    Output _PORT._PIN
    ' Clear all PORTS
    LATA = 0
    LATB = 0
    LATC = 0
    DelayMS 50
'*******************************************************************************
' Write some String in the Data eeprom memory.
E2p_AddressString1 EData "Testing the PIC18 AGV Bootloader V5.0 written in High Side of the ROM.",0

E2p_AddressString2 EData "New Universal PIC18 Bootloader by Alberto Freixanet. ",0

E2p_AddressString3 EData "A 8 digits Password could be needed to download the user code. ",0

'*******************************************************************************
STARTING_TEST:
'===============================================================================
' Configuration of the Timer0 to toggle a LED every 1.048Sec 
    ' Timer0 is configured As a 16-Bit timer/Counter, 1:256 prescale value: 1.048Sec 
    ' For xtal = 64
    INTCON2.7 = 1               ' All PORTB pull-ups are disabled.
    RCONBits_IPEN = 1           ' Enable priority levels on interrupts.
    IPR1Bits_TMR2IP = 1         ' Timer2 (Debounce Library) for High Interrupt.
    $if _xtal = 80
    T0CON = %00000111           ' /256 Timer0 is set for 0,8 second (more or less) with FOSC = 80Mhz.
    $endif
    $if _xtal = 64
    T0CON = %00000111           ' /256 Timer0 is set for 1 second (more or less) with FOSC = 64Mhz.
    $endif
    $if (_xtal > 16) And (_xtal <= 48)
    T0CON = %00000110           ' /128 Timer0 is set for 0,84 second (more or less) with FOSC = >16-48Mhz.
    $endif
    $if (_xtal = 16) Or (_xtal = 14)    
    T0CON = %00000101           ' /64 Timer0 is set for 1 second (more or less) with FOSC = 14-16Mhz.
    $endif
    $if (_xtal = 10) Or (_xtal = 12)    
    T0CON = %00000100           ' /32 Timer0 is set for 1 second (more or less) with FOSC = 12-10Mhz.
    $endif
    INTCON2Bits_TMR0IP = 0      ' Timer0 for Low Interrupt
    INTCONBits_TMR0IF = 0       ' Clear the Timer0 interrupt flag to begin.
    INTCONBits_TMR0IE = 1       ' Enable the Timer0 interrupts.
    INTCON.7 = 1                ' Enables all high priority interrupts
    INTCON.6 = 1                ' Enables all low priority interrupts
    TMR0H = 0                   ' Clear the Timer0 counter.
    Nop
    TMR0L = 0
    Nop    
    T0CONBits_TMR0ON = 1        ' Start the Timer0.
    PostScaler = 0              ' Not used for the moment.
'===============================================================================
    HRSOut CR,LF,CR,LF
'-------------------------------------------------------------------------------    
    HRSOut "USER CODE STARTS!",CR,LF 
'-------------------------------------------------------------------------------
' Read the WDT config in memory map from PPI file.
    EECON1 = %10000000              ' Access Flash memory
    Set_Bank TBLPTRL
    Asm
    Movlw Low (Config2H)
    Movwf TBLPTRL
    Movlw High (Config2H)
    Movwf TBLPTRH
    Movlw upper (Config2H)
    Movwf TBLPTRU
    EndAsm
    Tblrd*                          ' Read one memory position.  
    Temp1 = TABLAT                  ' Copy the TABLAT SFR immediatly because it is cleared if read out of this routine??
    EECON1 = 0                      ' Disable Access the Configuration registers
    TBLPTRU = 0                     ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
'-------------------------------------------------------------------------------    
    HRSOut "CONFIG FUSES: " 
    $if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22) Or (_device = _18F43K22) Or (_device = _18F44K22) Or (_device = _18F45K22) Or (_device = _18F46K22)
    Temp1 = Temp1 & 3
    If Temp1 = 3 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled."    
    ElseIf Temp1 = 2 Then
        HRSOut "WDT controlled by the SWDTEN bit."   
    ElseIf Temp1 = 1 Then
        HRSOut "WDT enabled when Device is active, disabled when Device is in Sleep; SWDTEN bit disabled."
    ElseIf Temp1 = 0 Then
        HRSOut "WDT disabled in hardware; SWDTEN bit disabled."
    EndIf
    $endif
    $if (_device = _18F242) Or (_device = _18F248) Or (_device = _18F252) Or (_device = _18F258) Or (_device = _18F442) Or (_device = _18F452) Or (_device = _18F458)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled."    
    Else
        HRSOut "WDT controlled by the SWDTEN bit."   
    EndIf
    $endif
    $if (_device = _18F2420) Or (_device = _18F2520) Or (_device = _18F4420) Or (_device = _18F4520)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled."    
    Else
        HRSOut "WDT controlled by the SWDTEN bit."   
    EndIf
    $endif
    $if (_device = _18F2525) Or (_device = _18F2620) Or (_device = _18F4525) Or (_device = _18F4620)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled."    
    Else
        HRSOut "WDT controlled by the SWDTEN bit."   
    EndIf
    $endif
    $if (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20) Or (_device = _18F43K20) Or (_device = _18F44K20) Or (_device = _18F45K20) Or (_device = _18F46K20)
    Temp1 = Temp1 & 1
    If Temp1 = 1 Then
        HRSOut "WDT enabled in hardware; SWDTEN bit disabled."    
    Else
        HRSOut "WDT controlled by the SWDTEN bit."   
    EndIf
    $endif
    HRSOut CR,LF
'-------------------------------------------------------------------------------
' Config Fuses:   BOREN = On    ;Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    If RCONBits_SBOREN = 1 Then
        HRSOut "Brown-out Reset enabled!",CR,LF 
    Else
        HRSOut "Brown-out Reset disabled!",CR,LF     
    EndIf
'-------------------------------------------------------------------------------
    HRSOut CR,LF
'===============================================================================
' Check the SWDTEN Bit.
    HRSOut "Be carrefull the WDT is always active!",CR,LF  
    If Temp1 = 0 Then
        HRSOut "After Reset the "
        If WDTCONBits_SWDTEN = 1 Then
            HRSOut "SWDTEN Bit = 1 (error)",CR,LF
        Else
            HRSOut "SWDTEN Bit = 0 (good, by code)",CR,LF
        EndIf
        WDTCONBits_SWDTEN = 1           ' Enable the Watchdog
        If WDTCONBits_SWDTEN = 1 Then
            HRSOut "Now the SWDTEN Bit is set => Watchdog Enabled.",CR,LF
        Else
            HRSOut "Now the SWDTEN Bit is Cleared => Watchdog Disabled.",CR,LF    
        EndIf
    EndIf
'-------------------------------------------------------------------------------
    ' Read the DEVID1 (Device ID) from the PPI file.
    EECON1 = %10000000               ' Access Flash memory
    Set_Bank TBLPTRL
    Asm
    Movlw Low (DEVID1)
    Movwf TBLPTRL
    Movlw High (DEVID1)
    Movwf TBLPTRH
    Movlw upper (DEVID1)
    Movwf TBLPTRU
    EndAsm
    Tblrd*+                         ' perform table read with post-increment
    DataWord.LowByte = TABLAT
    Tblrd*+                         ' perform table read with post-increment
    DataWord.HighByte = TABLAT        
    TBLPTRU = 0                     ' Cleared for security
    EECON1 = 0                      ' Disable Access the Configuration registers
    '---------------------------------------------------------------------------
    ' Calculate the PIC Part Number & revision
    PICPartNumber = DataWord >> 5
    PICRevision = DataWord.LowByte & 31
'-------------------------------------------------------------------------------
    ' Print the Device Name to the Terminal.
    $ifndef __DEVICE_ID
        $error "The Devicelist.inc is missing!!"
    $endif
    Select PICPartNumber
        Case __DEVICE_ID
            HRSOut "Device = ", __DEVICE_NAME, CR,LF
        Case Else
            HRSOut CR,LF    
            HRSOut "The PIC Device read from the Board is not correct!",CR,LF   
            HRSOut CR,LF        
    EndSelect    
'-------------------------------------------------------------------------------
    HRSOut "FOSC: ", Dec _xtal," Mhz"
    $ifdef _InternalOSC_
    HRSOut " Internal Oscillator ",CR,LF    
    $else
    HRSOut " Xtal Oscillator ",CR,LF    
    $endif
    HRSOut "PIC Part Number: ", Dec PICPartNumber, " / $", Hex4 PICPartNumber 
    HRSOut ", Revision: ", Dec PICRevision, " / $", Hex2 PICRevision,CR,LF

    $if PLL_ConfigFuses = On
        HRSOut "The PLL is used!",CR,LF
    $else
        $if PLL_ConfigFuses = OFF    
            HRSOut "FOSC without PLL!",CR,LF
        $endif
    $endif
    '---------------------------------------------------------------------------
    ' Read "_CWRITE_BLOCK" from asm file/PPI file.
    Set_Bank Temp1    
    Asm    
    Movlw (_CWRITE_BLOCK)  
    Movwf Temp1
    EndAsm    
    HRSOut "WRITE BLOCK: ", Dec Temp1," Bytes",CR,LF 
    '--------------------------------------------------------------------------- 
    HRSOut "ERASE BLOCK: ", Dec ERASE_FLASH_BLOCKSIZE," Bytes",CR,LF
    HRSOut "EEPROM MEMORY: ", Dec _eeprom," Bytes",CR,LF
    HRSOut "ROM MEMORY: ", Dec _code," Bytes",CR,LF     
    HRSOut "RAM MEMORY: ", Dec _ram," Bytes",CR,LF
    '---------------------------------------------------------------------------
    ' Read "RAM_BANKS" from asm file/PPI file.
    Set_Bank Temp1    
    Asm    
    Movlw (RAM_BANKS)  
    Movwf Temp1
    EndAsm
    HRSOut "RAM BANKS: ", Dec Temp1,CR,LF     
    '---------------------------------------------------------------------------
    ' Read "BankA_End" from asm file/PPI file.
    Set_Bank Temp1    
    Asm    
    Movlw (BankA_End)  
    Movwf Temp1
    EndAsm
    HRSOut "BANKA END: ", Dec Temp1,CR,LF      
    '---------------------------------------------------------------------------
    HRSOut "Number UART: 1 of ", Dec _uart,CR,LF 
    HRSOut "UART Baud Rate: ", Dec _BaudRate,CR,LF
'-------------------------------------------------------------------------------
    HRSOut CR,LF
'-------------------------------------------------------------------------------
' READING THE IDLOCS...
    HRSOut "Reading the PIC ID LOCs HEX format: "
    ' Read the IDLOC0 (ID Locations) from Asm file/PPI file.
    EECON1 = %10000000              ' Access memory flash 
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    For LoopByte = 0 To 7           ' Start the LoopByte                    
        Tblrd*+                     ' Read one memory position.
        HRSOut "$"
        HRSOut Hex2 TABLAT          ' Print results in hex format.
        HRSOut " "
    Next  
    EECON1 = 0                      ' Disable Access the Configuration registers
    TBLPTRU = 0
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    HRSOut "Reading the PIC ID LOCs DEC format: "
    ' Repeat loading the IDLOC0 address because the all address values have been
    ' modified by the HRSOut command.
    EECON1 = %10000000              ' Access memory Flash
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    For LoopByte = 0 To 7           ' Start the LoopByte
        Tblrd*+                     ' Read one memory position.
        HRSOut Dec TABLAT           ' Print results in decimal format.
        HRSOut " "
    Next      
    EECON1 = 0                      ' Disable Access the Configuration registers
    TBLPTRU = 0 ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
    HRSOut CR,LF
'-------------------------------------------------------------------------------
    HRSOut "Reading the PIC ID LOCs ASCII format: "
    ' Repeat loading the IDLOC0 address because the all address values have been
    ' modified by the HRSOut command.
    EECON1 = %10000000              ' Access memory Flash
    Set_Bank TBLPTRL
    Asm
    Movlw Low (IDLOC0)
    Movwf TBLPTRL
    Movlw High (IDLOC0)
    Movwf TBLPTRH
    Movlw upper (IDLOC0)
    Movwf TBLPTRU
    EndAsm
    NoASCII = 0
    For LoopByte = 0 To 7           ' Start the LoopByte
        Tblrd*+                     ' Read one memory position.
        If TABLAT > 32 And TABLAT < 127 Then 
            IDLOCS_String[LoopByte] = TABLAT    ' Load results in ASCII format.
        Else
            NoASCII = 1
            Break
        EndIf
    Next      
    EECON1 = 0  ' Disable Access the Configuration registers
    TBLPTRU = 0 ' The HRSOut command of the compiler does not clear the "TBLPTRU = 0", then take care of that.
                ' The HRSOut command can crash if the TBLPTRU SFR is not cleared.
    IDLOCS_String#8 = 0
    If NoASCII = 1 Then
        HRSOut "The ID Locations are not in ASCII Format"
    Else
        HRSOut IDLOCS_String
    EndIf
    HRSOut CR,LF
'-------------------------------------------------------------------------------
STRING_EEP:
    ' Send a String to the terminal read from the data eeprom memory.
    HRSOut CR,LF
    '---------------------------------------------------------------------------
    HRSOut "START EEPROM Strings:",CR,LF
    EepAddress = E2p_AddressString1
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString2
    GoSub ReadEEPString_Sub

    EepAddress = E2p_AddressString3
    GoSub ReadEEPString_Sub
    HRSOut "END EEPROM Strings:",CR,LF,CR,LF
'-------------------------------------------------------------------------------
    #ifdef __LOW_INTERRUPTS_ENABLED
    HRSOut "Low Interrupts are used!",CR,LF
    #endif
    #ifdef __HIGH_INTERRUPTS_ENABLED
    HRSOut "High Interrupts are used!",CR,LF
    #endif
    HRSOut CR,LF 
'-------------------------------------------------------------------------------
    HRSOut "The LED must toggle now!",CR,LF
    HRSOut CR,LF        
'===============================================================================
MAIN_CODE:
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        DelayMS 10
    Wend
'===============================================================================
'SUBROUTINES:
'===============================================================================
' Read a string written in the eeprom memory.
ReadEEPString_Sub:
    LoopByte = 0
    CounterEep = 0
    While 1 = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Temp1 = ERead EepAddress + LoopByte
        If Temp1 = 0 Then               ' Check  if final of the string.
            HRSOut CR,LF
            Return 
        EndIf
        If Temp1 = $FF Then             ' Check if this address has been erased.
            Inc CounterEep
            If CounterEep > 20 Then     ' Too much addresses erased.
                HRSOut "This String is erased!",CR,LF
                Return     
            EndIf
        Else
            HRSOut Temp1
        EndIf
        Inc LoopByte
    Wend 
'===============================================================================
    Stop
'===============================================================================
' Write some static data to the ID Locations.
'Asm
'__IDLOCS   0x200000,0x08
'__IDLOCS   0x200001,0x07
'__IDLOCS   0x200002,0x06
'__IDLOCS   0x200003,0x05
'__IDLOCS   0x200004,0x04
'__IDLOCS   0x200005,0x03
'__IDLOCS   0x200006,0x02
'__IDLOCS   0x200007,0x01
'EndAsm

'Asm
'__IDLOCS   IDLOC0,"L"
'__IDLOCS   IDLOC1,"O"
'__IDLOCS   IDLOC2,"A"
'__IDLOCS   IDLOC3,"D"
'__IDLOCS   IDLOC4,"E"
'__IDLOCS   IDLOC5,"R"
'__IDLOCS   IDLOC6,"5"
'__IDLOCS   IDLOC7,"0"
'EndAsm

'===============================================================================
